local addon = LibStub("AceAddon-3.0"):NewAddon("AO_BossMod", "AceEvent-3.0")
local version="1.2"

local addonPrefix = "AOBossMod"
local lastPlayedTime = {}
local silenceDelay = 4

local debugMode = false

-- Bibliothèque de sons
local mediaPath="Interface\\AddOns\\ao_bossmod\\medias\\"
local soundsTable = {
	mannoroth_felseeker="mannoroth_felseeker.ogg",
    mannoroth_gaze="mannoroth_gaze.ogg",
    mannoroth_fel_implosion="mannoroth_fel_implosion.ogg",
	mannoroth_shadowforce="mannoroth_shadowforce.ogg",
	archimonde_danse="archimonde_danse.ogg",
	archimonde_marque="archimonde_marque.ogg",
	archimonde_conduit="archimonde_conduit.ogg",
	archimonde_tourment="archimonde_tourment.ogg"
}

-- Menu de configuration
local soundToTest
local options = {
  type = "group",
  args = {
    enable = {
      name = "Enable AO BossMod",
      desc = "Enables / disables AO BossMod",
      type = "toggle",
			width = "full",
      set = function(info,val) addon.db.profile.enabled = val end,
      get = function(info) return addon.db.profile.enabled end
    },
		soundChannel = {
			name = "Sound Channel",
			desc = "Enter sound channel for voice call",
			type = "select",
			order = 1,
			values = {
				Master = "Master",
				SFX = "SFX",
				Ambience = "Ambience",
				Music = "Music"
			},
			set = function(info,val) addon.db.profile.soundchannel = val end,
			get = function(info) return addon.db.profile.soundchannel end
		},
		testVoice = {
			name = "Test Voice",
			desc = "Test voice call (select the sound to test, and click the GO button)",
			type = "select",
			order = 2,
			values = soundsTable,
			set = function(info,val) soundToTest = val end,
			get = function(info) return soundToTest end
		},
		goTestVoice = {
			name = "GO!",
			desc = "Play Sound File!",
			type = "execute",
			order = 3,
			--func = function() if soundToTest then	addon:handleRaidInstruction(soundToTest, "Test Button") end end
			func = function() if soundToTest then PlaySoundFile(mediaPath..soundsTable[soundToTest], addon.db.profile.soundchannel) end end
		},
    mannoroth={
      name = "HFC : Mannoroth",
      type = "group",
      args={
				felseeker = {
		      name = "Felseeker",
		      desc = "Enables / disables felseeker alert",
		      type = "toggle",
					width = "full",
		      set = function(info,val) addon.db.profile.enabledFelseeker = val end,
		      get = function(info) return addon.db.profile.enabledFelseeker end
		    },
				enabledGaze = {
		      name = "Mannoroth Gaze",
		      desc = "Enables / disables Mannoroth Gaze Alert",
		      type = "toggle",
					width = "full",
		      set = function(info,val) addon.db.profile.enabledGaze = val end,
		      get = function(info) return addon.db.profile.enabledGaze end
		    },
				enabledFelImplosion = {
		      name = "Fel Implosion",
		      desc = "Enables / disables Fel Implosion Alert",
		      type = "toggle",
					width = "full",
		      set = function(info,val) addon.db.profile.enabledFelImplosion = val end,
		      get = function(info) return addon.db.profile.enabledFelImplosion end
		    },
				enabledShadowforce = {
		      name = "Empowered Shadow Force",
		      desc = "Enables / disables Empowered Shadow Force",
		      type = "toggle",
					width = "full",
		      set = function(info,val) addon.db.profile.enabledShadowforce = val end,
		      get = function(info) return addon.db.profile.enabledShadowforce end
		    }
      }
	},
	archimonde={
      name = "HFC : Archimonde",
      type = "group",
      args={
			enabledShackledTorment = {
		      name = "Shackled Torment Countdown",
		      desc = "Enables / disables Shackled Torment countdown",
		      type = "toggle",
			  width = "full",
		      set = function(info,val) addon.db.profile.enabledShackledTorment = val end,
		      get = function(info) return addon.db.profile.enabledShackledTorment end
		    },
			enabledDarkConduit = {
		      name = "Dark Conduit",
		      desc = "Enables / disables Dark Conduit alert",
		      type = "toggle",
			  width = "full",
		      set = function(info,val) addon.db.profile.enabledDarkConduit = val end,
		      get = function(info) return addon.db.profile.enabledDarkConduit end
		    },
			enabledMarkLegion = {
		      name = "Mark of the Legion",
		      desc = "Enables / disables Mark of the Legion Alert",
		      type = "toggle",
			  width = "full",
		      set = function(info,val) addon.db.profile.enabledMarkLegion = val end,
		      get = function(info) return addon.db.profile.enabledMarkLegion end
		    },
			enabledSeethingCorruption = {
		      name = "Seething Corruption",
		      desc = "Enables / disables Seething Corruption Alert",
		      type = "toggle",
			  width = "full",
		      set = function(info,val) addon.db.profile.enabledSeethingCorruption = val end,
		      get = function(info) return addon.db.profile.enabledSeethingCorruption end
		    }
      }
	}
  }
}

-- Valeurs par défaut
local defaults = {
	profile = {
		enabled = true,
		enabledFelImplosion = true,
		enabledFelseeker = true,
		enabledGaze = true,
		enabledShadowforce = true,
		enabledShackledTorment = true,
		enabledDarkConduit = true,
		enabledMarkLegion = true,
		enabledSeethingCorruption = true,
		soundchannel = "Master"
	}
}

local function printDebug(text)
	if debugMode then print("[DEBUG][CLIENT] "..text) end
end

function addon:OnInitialize()
    -- Called when the addon is loaded
		self.db = LibStub("AceDB-3.0"):New("AO_BossModDB", defaults, true)
		LibStub("AceConfig-3.0"):RegisterOptionsTable("AO_BossMod", options)
    self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("AO_BossMod", "AO BossMod [Player]")
end

function addon:OnEnable()
	printDebug("AO Boss Mod Player init")
	self:RegisterEvent("CHAT_MSG_ADDON","handleMessageAddon")
	RegisterAddonMessagePrefix(addonPrefix)
end

-- addon:CHAT_MSG_ADDON
-- Executé lors d'un évenement CHAT_MSG_ADDON
-- Vérifie qu'il s'agit d'un message AOBossMod, et que l'émetteur est un assistant ou le raid leader
function addon:handleMessageAddon(event,prefix,msg,channel,sender)
  --printDebug(event..prefix..msg..channel..sender)
	if prefix == addonPrefix and (UnitIsGroupAssistant(sender) or UnitIsGroupLeader(sender)) then
		if strfind(msg,"myAOBossModVersion") then return end -- pour éviter de traiter son propre wisp de réponse...
		if strfind(msg,"getAOBossModVersion") then
			self:handleGetVersion(sender)
		else
			if (self.db.profile.enabled) then self:handleRaidInstruction(msg,sender) end
		end
  end
end

-- handleRaidInstruction
-- Lance la lecture de l'instruction vocale envoyé par le raid lead
-- Zone de "silence" (variable silenceDelay) après chaque lecture, si jamais plusieurs assistants ont le module raid lead, ça évite de flood.
function addon:handleRaidInstruction(message,sender)
  if (not self.db.profile.enabledGaze and message=="mannoroth_gaze") then return end
	-- Ici c'est laid, faut refaire un truc mieux que ça bobby...
	if (not self.db.profile.enabledFelseeker and message=="mannoroth_felseeker") then return end
	if (not self.db.profile.enabledFelImplosion and message=="mannoroth_fel_implosion") then return end
	if (not self.db.profile.enabledShadowforce and message=="mannoroth_shadowforce") then return end
	if (not self.db.profile.enabledShackledTorment and message=="archimonde_tourment") then return end
	if (not self.db.profile.enabledDarkConduit and message=="archimonde_conduit") then return end
	if (not self.db.profile.enabledMarkLegion and message=="archimonde_marque") then return end
	if (not self.db.profile.enabledSeethingCorruption and message=="archimonde_danse") then return end

	printDebug("Receive message "..message.." from "..sender)
	if (lastPlayedTime[message]==nil) or (time()-lastPlayedTime[message]>silenceDelay) then
		if not soundsTable[message] then print("AO Boss Mod : Attention, votre version d'AO BossMod n'est pas à jour.") end
		lastPlayedTime[message]=time()
		printDebug("Play sound "..mediaPath..soundsTable[message].." on "..self.db.profile.soundchannel)
		PlaySoundFile(mediaPath..soundsTable[message], self.db.profile.soundchannel)
	end
end

-- handleGetVersion
-- Le raid leader a envoyé une requête pour connaître la version du client
function addon:handleGetVersion(sender)
	printDebug("Receive message getAOBossModVersion from "..sender)
	printDebug("Send Version "..version.." to "..sender)
	SendAddonMessage(addonPrefix, "myAOBossModVersion "..version, "WHISPER", sender)
end
